"""
Copyright 2016-2017 Hermann Krumrey

This file is part of xdcc-dl.

xdcc-dl is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

xdcc-dl is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with xdcc-dl.  If not, see <http://www.gnu.org/licenses/>.
"""


class BotChannelMapper(object):
    """
    Class that maps bots with non-standard whois responses to a channel
    """

    bot_channel_map = {"HelloKitty": "#horriblesubs"}

    @staticmethod
    def has_mapping(botname: str) -> bool:
        """
        Checks if a bot is mapped to a channel in this class

        :param botname: The bot's name
        :return:        True, if it has a mapping, false otherwise
        """
        return botname in BotChannelMapper.bot_channel_map.keys()

    @staticmethod
    def map(botname: str) -> str:
        """
        Maps a bot to a channel name

        :param botname: The bot's name
        :return:        The mapped channel name
        """
        return BotChannelMapper.bot_channel_map[botname]



"""
Copyright 2016-2017 Hermann Krumrey

This file is part of xdcc-dl.

xdcc-dl is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

xdcc-dl is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with xdcc-dl.  If not, see <http://www.gnu.org/licenses/>.
"""

# imports
import irc.client
import irc.events
# noinspection PyUnresolvedReferences,PyPep8Naming
from xdcc_dl.logging.LoggingTypes import LoggingTypes as LOG
from xdcc_dl.xdcc.layers.irc.BaseIrcClient import BaseIrclient


def create_event_printer(event: str, formatting: str, show_tag: bool = False) \
        -> str:
    """
    Creates a new method definition for an on_event method
    that can then be integrated using exec

    :param event:      The name of the event on which this method is called
    :param formatting: The formatting from LOG, but as a string
    :param show_tag:   Determines if an 'on_event" string should be
                       printed beforehand
    :return:           The method as an executable string
    """
    method_definition = "def on_" + event + "(self, connection, event):\n"
    if show_tag:
        method_definition += "    self.logger.log(\"on_" + event + \
                             "\", LOG.EVENT)\n"
    method_definition += "    print_args = \"\"\n"
    method_definition += "    for arg in event.arguments:\n"
    method_definition += "        print_args += arg + \" \"\n"
    method_definition += "    print_args = print_args.rstrip().lstrip()\n"
    method_definition += "    self.logger.log(print_args, " + formatting + ")"
    return method_definition


# noinspection PyMethodMayBeStatic,PyUnusedLocal
class IrcEventPrinter(BaseIrclient):
    """
    Class that prints output to the console for every type of IRC event
    defined by the irc library.
    Layer 1 of the XDCC Bot
    """

    # This assigns a method to all event types that
    # are supported by the IRC library.
    # They print the type of event and its parameters
    for event in irc.events.all:
        if event != "disconnect":
            method = create_event_printer(event, "LOG.EVENT_TEXT", True)
            exec(method)

    # Here are some more specialized ones

    # Prints the server greeting
    for event in [
        "welcome",
        "yourhost",
        "created",
        "myinfo",
        "featurelist",
        "luserclient",
        "luserop",
        "luserunknown",
        "luserchannels",
        "luserme",
        "n_local",
        "n_global",
        "modstart"
    ]:
        exec(create_event_printer(event, "LOG.WELCOME", False))

    # Message of the Day
    for event in ["motd", "motdstart", "endofmotd"]:
        exec(create_event_printer(event, "LOG.MESSAGE_OF_THE_DAY", False))

    # Here we manually define the more interesting methods:

    def on_privnotice(self, connection: irc.client.ServerConnection,
                      event: irc.client.Event):
        """
        Prints Private Notices

        :param connection: the IRC Connection
        :param event:      the IRC Event
        :return:           None
        """
        self.logger.log(event.arguments[0], LOG.PRIVATE_NOTICE)

    def on_privmsg(self, connection: irc.client.ServerConnection,
                   event: irc.client.Event):
        """
        Prints Private Messages

        :param connection: the IRC Connection
        :param event:      the IRC Event
        :return:           None
        """
        self.logger.log(event.source + ": " + event.arguments[0],
                        LOG.PRIVATE_MESSAGE)

    def on_pubnotice(self, connection: irc.client.ServerConnection,
                     event: irc.client.Event):
        """
        Prints Public Notices

        :param connection: the IRC Connection
        :param event:      the IRC Event
        :return:           None
        """
        self.logger.log(event.source + ": " + event.arguments[0],
                        LOG.PUBLIC_NOTICE)

    def on_pubmsg(self, connection: irc.client.ServerConnection,
                  event: irc.client.Event):
        """
        Prints Public Messages

        :param connection: the IRC Connection
        :param event:      the IRC Event
        :return:           None
        """
        self.logger.log(event.source + ": " + event.arguments[0],
                        LOG.PUBLIC_MESSAGE)

    def on_ping(self, connection: irc.client.ServerConnection,
                event: irc.client.Event):
        """
        Prints Pings

        :param connection: the IRC Connection
        :param event:      the IRC Event
        :return:           None
        """
        self.logger.log("PING: " + event.arguments[0], LOG.PING)

    def on_ctcp(self, connection: irc.client.ServerConnection,
                event: irc.client.Event):
        """
        Prints Client-To-Client_Protocol 'Version' Messages

        :param connection: the IRC Connection
        :param event:      the IRC Event
        :return:           None
        """
        if event.arguments[0] == "VERSION":
            self.logger.log(event.arguments[0], LOG.CTCP_VERSION)
            return



"""
Copyright 2016-2017 Hermann Krumrey

This file is part of xdcc-dl.

xdcc-dl is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

xdcc-dl is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with xdcc-dl.  If not, see <http://www.gnu.org/licenses/>.
"""

# imports
import os
import unittest
from xdcc_dl.entities.XDCCPack import XDCCPack
from xdcc_dl.entities.Progress import Progress
from xdcc_dl.entities.IrcServer import IrcServer
from xdcc_dl.xdcc.MultipleServerDownloader import MultipleServerDownloader


class UnitTests(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        if os.path.isfile("1_test.txt"):
            os.remove("1_test.txt")
        if os.path.isfile("2_test.txt"):
            os.remove("2_test.txt")
        if os.path.isfile("3_test.txt"):
            os.remove("3_test.txt")

    def test_download_multiple_packs_same_server(self):

        progress = Progress(2)

        downloader = MultipleServerDownloader("random")

        downloader.download([
            XDCCPack(IrcServer("irc.namibsun.net"), "xdcc_servbot", 1),
            XDCCPack(IrcServer("irc.namibsun.net"), "xdcc_servbot", 3)
        ], progress)

        self.assertTrue(os.path.isfile("1_test.txt"))
        self.assertTrue(os.path.isfile("3_test.txt"))

        self.assertEqual(progress.get_single_progress_percentage(), 100.0)
        self.assertEqual(progress.get_total_percentage(), 100.0)

    def test_download_multiple_packs_different_servers(self):

        progress = Progress(2)

        downloader = MultipleServerDownloader("random")

        downloader.download([
            XDCCPack(IrcServer("irc.namibsun.net"), "xdcc_servbot", 2),
            XDCCPack(IrcServer("namibsun.net"), "xdcc_servbot", 3)
        ], progress)

        self.assertTrue(os.path.isfile("2_test.txt"))
        self.assertTrue(os.path.isfile("3_test.txt"))

        self.assertEqual(progress.get_single_progress_percentage(), 100.0)
        self.assertEqual(progress.get_total_percentage(), 100.0)

    def test_quitting(self):

        packs = [XDCCPack(IrcServer("irc.namibsun.net"), "xdcc_servbot", 1)]

        downloader = MultipleServerDownloader("random")
        downloader.quit()
        downloader.download(packs)
        downloader.quit()

        self.assertFalse(os.path.isfile("1_test.txt"))



"""
Copyright 2016-2017 Hermann Krumrey

This file is part of xdcc-dl.

xdcc-dl is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

xdcc-dl is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with xdcc-dl.  If not, see <http://www.gnu.org/licenses/>.
"""

# imports
import os
import sys
import argparse
from xdcc_dl.xdcc.XDCCDownloader import XDCCDownloader
from xdcc_dl.tui.XDCCDownloaderTui import XDCCDownloaderTui
from xdcc_dl.entities.XDCCPack import xdcc_packs_from_xdcc_message

try:
    from xdcc_dl.gui.XDCCDownloaderGui import start as start_gui
except ImportError:  # pragma: no cover
    start_gui = None


def main():
    """
    Starts the main method of the program

    :return: None
    """
    try:

        parser = argparse.ArgumentParser()
        parser.add_argument("message", nargs='?',
                            help="An XDCC Message. "
                                 "Supports ranges (1-100) and "
                                 "also ranges with steps (1-100,2)")
        parser.add_argument("-s", "--server",
                            help="Specifies the IRC Server. "
                                 "Defaults to irc.rizon.net")
        parser.add_argument("-d", "--destination",
                            help="Specifies the target download destination. "
                                 "Defaults to " + os.getcwd())
        parser.add_argument("-u", "--username",
                            help="Specifies the username")
        parser.add_argument("-v", "--verbosity", type=int, default=1,
                            help="Specifies the verbosity of the output "
                                 "on a scale of 1-7. Default: 1")
        parser.add_argument("-g", "--gui", action="store_true",
                            help="Starts the XDCC Downloader GUI")
        parser.add_argument("-t", "--tui", action="store_true",
                            help="Starts the XDCC Downloader TUI")
        args = parser.parse_args()

        if args.message:

            destination = os.getcwd() if not args.destination \
                else args.destination

            server = "irc.rizon.net" if not args.server else args.server
            user = "random" if not args.username else args.username
            verbosity = args.verbosity

            packs = xdcc_packs_from_xdcc_message(
                args.message, destination, server
            )

            downloader = XDCCDownloader(server, user, verbosity)
            results = downloader.download(packs)
            downloader.quit()

            max_length = max(map(
                lambda x: len(x.get_filepath()), results.keys()
            ))
            for result in results:
                print_result = result.get_filepath().ljust(max_length) + " - "
                print_result += results[result]
                print(print_result)

        elif args.gui:  # pragma: no cover
            if start_gui is not None:
                start_gui()
            else:
                print("Error: PyQt5 not installed")

        elif args.tui:  # pragma: no cover
            XDCCDownloaderTui().start()

        else:
            print("No arguments passed. See --help for more details")
            sys.exit(0)

    except KeyboardInterrupt:
        print("Thanks for using xdcc-dl!")


if __name__ == "__main__":  # pragma: no cover

    if sys.platform == "win32" and len(sys.argv) == 1:
        sys.argv.append("-g")
    main()



"""
Copyright 2016-2017 Hermann Krumrey

This file is part of xdcc-dl.

xdcc-dl is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

xdcc-dl is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with xdcc-dl.  If not, see <http://www.gnu.org/licenses/>.
"""

# imports
import os
import unittest
from xdcc_dl.pack_searchers.procedures.ixirc import find_ixirc_packs


class UnitTests(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_empty_query(self):
        packs = find_ixirc_packs("")
        self.assertEqual(len(packs), 0)

    def test_gin_txt(self):
        packs = find_ixirc_packs("Gin.txt")
        self.assertEqual(len(packs), 1)

        pack = packs[0]
        self.assertEqual(pack.get_packnumber(), 1)
        self.assertEqual(pack.get_server().get_address(), "irc.abjects.net")
        self.assertEqual(pack.get_filepath(),
                         os.path.join(os.getcwd(), pack.get_filename()))
        self.assertEqual(pack.get_filename(), "Gin.txt")
        self.assertEqual(pack.get_bot(), "Beast-Gin-Anime")

    def test_larger_result(self):
        packs = find_ixirc_packs("game of thrones")
        self.assertLess(10, len(packs))

    def test_non_result_query(self):
        self.assertEqual(
            0,
            len(find_ixirc_packs(
                "sdgyfdhkdashsahdqhdsadlsajdhsaohdsausahoashdsahdlahdsah"
            ))
        )

    def test_invalid_pack_handling(self):
        self.assertLess(10, len(find_ixirc_packs("Star Wars Episode 7")))



"""
Copyright 2016-2017 Hermann Krumrey

This file is part of xdcc-dl.

xdcc-dl is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

xdcc-dl is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with xdcc-dl.  If not, see <http://www.gnu.org/licenses/>.
"""

# imports
import unittest
from xdcc_dl.xdcc.layers.xdcc.XDCCInitiator import XDCCInitiator


class TestException(Exception):
    pass


class UnitTests(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_dcc_check(self):

        class Tester(XDCCInitiator):

            def on_welcome(self, conn, event):
                event.arguments = ["NOTDCC"]
                self.on_ctcp(conn, event)
                raise TestException()

        try:
            Tester("irc.namibsun.net", "random").start()
            self.assertTrue(False)
        except TestException:
            self.assertTrue(True)

    # HelloKitty no longer exists
    # def test_missing_whois(self):
    #    BotChannelMapper.bot_channel_map = {}
    #
    #    try:
    #        initiator = XDCCInitiator("irc.rizon.net", "random")
    #        initiator.current_pack = \
    #            XDCCPack(IrcServer("irc.rizon.net"), "HelloKitty", 1)
    #        initiator.start()
    #        self.assertTrue(False)
    #    except NoValidWhoisQueryException:
    #        self.assertTrue(True)



"""
Copyright 2016-2017 Hermann Krumrey

This file is part of xdcc-dl.

xdcc-dl is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

xdcc-dl is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with xdcc-dl.  If not, see <http://www.gnu.org/licenses/>.
"""
# imports
import sys
import unittest
from xdcc_dl.main import main


class UnitTests(unittest.TestCase):

    def setUp(self):
        sys.argv = [sys.argv[0]]

    def tearDown(self):
        sys.argv = [sys.argv[0]]

    def test_no_arguments(self):
        try:
            main()
            self.assertEqual(True, False)
        except SystemExit:
            self.assertEqual(True, True)

    # noinspection PyStatementEffect
    def test_keyboard_interrupt(self):

        if sys.version_info[0] >= 3:
            exec("import builtins as __builtin__") in globals()

            # noinspection PyUnusedLocal
            def interrupt(arg):
                if arg == "No arguments passed. See --help for more details":
                    raise KeyboardInterrupt()
                else:
                    self.assertEqual("Thanks for using xdcc-dl!", arg)

            exec("real_print = __builtin__.print") in globals()
            exec("__builtin__.print = interrupt") in globals()

            main()

            exec("__builtin__.print = real_print") in globals()



"""
Copyright 2016-2017 Hermann Krumrey

This file is part of xdcc-dl.

xdcc-dl is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

xdcc-dl is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with xdcc-dl.  If not, see <http://www.gnu.org/licenses/>.
"""

# imports
import unittest
from xdcc_dl.xdcc.layers.irc.BaseIrcClient import BaseIrclient, NetworkError


class TestException(Exception):
    pass


class UnitTests(unittest.TestCase):

    def setUp(self):
        self.client = BaseIrclient

    def tearDown(self):
        self.client.quit()

    def test_faulty_server(self):

        self.client = BaseIrclient("gitlab.namibsun.net", "random")
        try:
            self.client.start()
            self.assertTrue(False)
        except NetworkError as e:
            self.assertEqual(str(e), "Failed to connect to Server")

    def test_server_connect(self):

        class Tester(BaseIrclient):

            # noinspection PyMethodMayBeStatic
            def on_welcome(self, conn, event):
                raise TestException()

        self.client = Tester("irc.namibsun.net", "random")

        try:
            self.client.start()
            self.assertTrue(False)
        except TestException:
            self.assertTrue(True)

    def test_on_banned(self):

        class Tester(BaseIrclient):
            def on_welcome(self, conn, event):
                self.on_error(conn, event)

        self.client = Tester("irc.namibsun.net", "random")

        try:
            self.client.start()
            self.assertTrue(False)
        except NetworkError as e:
            self.assertEqual(str(e), "Failed to connect due to a ban")



"""
Copyright 2016-2017 Hermann Krumrey

This file is part of xdcc-dl.

xdcc-dl is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

xdcc-dl is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with xdcc-dl.  If not, see <http://www.gnu.org/licenses/>.
"""

# imports
from typing import List, Dict

from xdcc_dl.logging import Logger
from xdcc_dl.entities.User import User
from xdcc_dl.entities.XDCCPack import XDCCPack
from xdcc_dl.entities.Progress import Progress
from xdcc_dl.entities.IrcServer import IrcServer
from xdcc_dl.xdcc.XDCCDownloader import XDCCDownloader


class MultipleServerDownloader(object):
    """
    Class that can handle downloading from various server sources
    """

    def __init__(self, user: User or str, logger: Logger or int = 0):
        """
        Creates a new multiple server downloader, without specifying the server

        :param user:    The user with which to log in
        :param logger:  The logger to use
        """
        self.user = user
        self.logger = logger
        self.current_downloader = None
        self.quit_called = False

    def download(self, packs: List[XDCCPack], progress: Progress = None)\
            -> Dict[XDCCPack, str]:
        """
        Downloads all XDCC packs specified. Optionally shares state with other
        threads using a Progress object

        :param packs: The packs to download
        :param progress: Optional Progress object
        :return: Dictionary of packs mapped to status codes:
                   "OK":           Download was successful
                   "BOTNOTFOUND":  Bot was not found
                   "NETWORKERROR": Download failed due to network error
                   "INCOMPLETE":   Download was incomplete
                   "EXISTED":      File already existed and
                                   was completely downloaded
        """
        results = {}

        packservers = {}

        for pack in packs:
            try:
                packservers[pack.get_server().get_address()].append(pack)
            except KeyError:
                packservers[pack.get_server().get_address()] = [pack]

        for server in packservers:

            if not self.quit_called:
                self.current_downloader = \
                    XDCCDownloader(IrcServer(server), self.user, self.logger)
                server_results = \
                    self.current_downloader.download(
                        packservers[server], progress
                    )

                for result in server_results:
                    results[result] = server_results[result]

        return results

    def quit(self):
        """
        Quits the current downloader and stops all downloads to come

        :return: None
        """
        if self.current_downloader is not None:
            self.current_downloader.quit()
        self.quit_called = True



"""
Copyright 2016-2017 Hermann Krumrey

This file is part of xdcc-dl.

xdcc-dl is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

xdcc-dl is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with xdcc-dl.  If not, see <http://www.gnu.org/licenses/>.
"""

# imports
import unittest
from xdcc_dl.entities.User import User


class UnitTests(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_user_constructor(self):

        user = User("username")
        self.assertEqual(user.get_name(), "username")

    def test_random_username_generation(self):

        user_one = User("random")
        user_two = User()
        user_three = User("")

        self.assertNotEqual("random", user_one.get_name())
        self.assertNotEqual("", user_two.get_name())
        self.assertNotEqual("", user_three.get_name())

        self.assertNotEqual(user_one.get_name(), user_two.get_name())
        self.assertNotEqual(user_two.get_name(), user_three.get_name())
        self.assertNotEqual(user_three.get_name(), user_one.get_name())



"""
Copyright 2016-2017 Hermann Krumrey

This file is part of xdcc-dl.

xdcc-dl is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

xdcc-dl is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with xdcc-dl.  If not, see <http://www.gnu.org/licenses/>.
"""

# imports
import os
from xdcc_dl import version
from setuptools import setup, find_packages


def readme():
    """
    Reads the readme file and converts it to RST if pypandoc is
    installed. If not, the raw markdown text is returned
    :return: the readme file as a string
    """
    # noinspection PyBroadException
    try:
        # noinspection PyPackageRequirements,PyUnresolvedReferences
        import pypandoc
        with open("README.md") as f:
            # Convert markdown file to rst
            markdown = f.read()
            rst = pypandoc.convert(markdown, "rst", format="md")
            return rst

    except ModuleNotFoundError:
        # If pandoc is not installed, just return the raw markdown text
        with open("README.md") as f:
            return f.read()


def find_scripts():
    """
    Returns a list of scripts in the bin directory
    :return: the list of scripts
    """
    scripts = []

    for file_name in os.listdir("bin"):

        path = os.path.join("bin", file_name)
        if file_name == "__init__.py":
            continue
        elif not os.path.isfile(path):
            continue
        else:
            scripts.append(os.path.join("bin", file_name))

    return scripts


setup(
    name="xdcc_dl",
    version=version,
    description="An XDCC File Downloader based on the irclib framework",
    long_description=readme(),
    classifiers=[
        "Environment :: Console",
        "Natural Language :: English",
        "Intended Audience :: Developers",
        "Development Status :: 1 - Planning",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 2",
        "Topic :: Communications :: File Sharing",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)"
    ],
    url="https://gitlab.namibsun.net/namboy94/xdcc-dl",
    download_url="https://gitlab.namibsun.net/namboy94/xdcc-dl/"
                 "repository/archive.zip?ref=master",
    author="Hermann Krumrey",
    author_email="hermann@krumreyh.com",
    license="GNU GPL3",
    packages=find_packages(),
    install_requires=["irc", "bs4", "requests", "cfscrape", "urwid", "typing"],
    test_suite='nose.collector',
    tests_require=['nose'],
    scripts=find_scripts(),
    zip_safe=False
)



"""
Copyright 2016-2017 Hermann Krumrey

This file is part of xdcc-dl.

xdcc-dl is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

xdcc-dl is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with xdcc-dl.  If not, see <http://www.gnu.org/licenses/>.
"""

# imports
from typing import List, Dict

from xdcc_dl.entities.XDCCPack import XDCCPack
from xdcc_dl.entities.Progress import Progress
from xdcc_dl.xdcc.layers.irc.BaseIrcClient import NetworkError
from xdcc_dl.xdcc.layers.irc.BotFinder import BotNotFoundException
from xdcc_dl.xdcc.layers.xdcc.DownloadHandler import DownloadHandler,\
    AlreadyDownloaded
from xdcc_dl.xdcc.layers.xdcc.XDCCInitiator import IncorrectFileSentException,\
    NoValidWhoisQueryException


class XDCCDownloader(DownloadHandler):
    """
    The XDCC Downloader that combines the capabilities of
    all XDCC Layers to offer a stable interface to download XDCC Packs
    """

    def download(self, packs: List[XDCCPack], progress: Progress = None)\
            -> Dict[XDCCPack, str]:
        """
        Downloads all XDCC packs specified. Optionally shares state with other
        threads using a Progress object.
        All packs need to connect to the same server

        :param packs:    The packs to download
        :param progress: Optional Progress object
        :return: Dictionary of packs mapped to status codes:
                   "OK":              Download was successful
                   "BOTNOTFOUND":     Bot was not found
                   "CHANNELJOINFAIL": Channel join failed, most likely due to
                                      missing whois information
                   "NETWORKERROR":    Download failed due to network error
                   "INCORRECT":       Sent file was not the correct file
                   "EXISTED":         File already existed and was
                                      completely downloaded
                   "OTHERSERVER":     If a pack was found that is hosted on a
                                      different server
        """
        self.progress = progress if progress is not None else\
            Progress(len(packs))
        self.pack_queue = packs
        self.pack_states = {}

        while len(self.pack_queue) > 0:

            self.current_pack = self.pack_queue.pop(0)

            if self.current_pack.get_server().get_address() != \
                    self.server.get_address():
                self.pack_states[self.current_pack] = "OTHERSERVER"
                continue

            status_code = "OK"

            try:
                self.start()
            except BotNotFoundException:
                status_code = "BOTNOTFOUND"
            except NoValidWhoisQueryException:
                status_code = "CHANNELJOINFAIL"
            except AlreadyDownloaded:
                status_code = "EXISTED"
            except IncorrectFileSentException:
                status_code = "INCORRECT"
            except NetworkError:
                status_code = "NETWORKERROR"

            self.pack_states[self.current_pack] = status_code
            self.reset_connection_state()
            self.progress.next_file()

        self.quit()

        return self.pack_states



"""
Copyright 2016-2017 Hermann Krumrey

This file is part of xdcc-dl.

xdcc-dl is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

xdcc-dl is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with xdcc-dl.  If not, see <http://www.gnu.org/licenses/>.
"""

# imports
import unittest
from xdcc_dl.pack_searchers.PackSearcher import PackSearcher


class UnitTests(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_all_searchers(self):

        # Subject to change when real-world stuff changes
        for term, amount in {
            "AHQ Official Dragon Ball Z Releases.txt": 2,
            "Gin.txt": 1,
            "mashiro.txt": 1,
            "1_test.txt": 1
        }.items():
            results = PackSearcher().search(term)
            self.assertEqual(len(results), amount)

    def test_getting_searchers(self):

        procedures = PackSearcher.get_available_pack_searchers()
        searcher = PackSearcher()

        for procedure in procedures:
            self.assertTrue(PackSearcher.procedure_map[procedure]
                            in searcher.procedures)

    def test_selected_searchers(self):

        searcher = PackSearcher(["nibl", "namibsun"])
        self.assertEqual(len(searcher.search(
            "AHQ Official Dragon Ball Z Releases.txt")
        ), 1)

        for procedure in PackSearcher.get_available_pack_searchers():

            if procedure not in ["nibl", "namibsun"]:
                self.assertFalse(PackSearcher.procedure_map[procedure]
                                 in searcher.procedures)



"""
Copyright 2016-2017 Hermann Krumrey

This file is part of xdcc-dl.

xdcc-dl is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

xdcc-dl is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with xdcc-dl.  If not, see <http://www.gnu.org/licenses/>.
"""

# imports
import irc.client
from xdcc_dl.xdcc.layers.irc.BotFinder import BotFinder
# noinspection PyPep8Naming
from xdcc_dl.logging.LoggingTypes import LoggingTypes as LOG


# noinspection PyUnusedLocal
class MessageSender(BotFinder):
    """
    Class that handles initiating the XDCC Connection by sending the
    appropriate message to the bot.
    Layer 3 of the XDCC Bot
    """

    def on_join(self, connection: irc.client.ServerConnection,
                event: irc.client.Event):
        """
        Method called when a channel was joined.
        The first channel that was joined triggers sending a message
        to the bot to initiate the XDCC handshake

        :param connection: the IRC Connection
        :param event:      the IRC Event
        :return:           None
        """
        # Check if we were the ones joining
        if event.source.startswith(self.user.get_name()):
            if not self.user.get_name() == event.target and \
                    self.channel_join_required:
                # Check if we didn't ask a channelless bot
                self.logger.log("Joined Channel " + event.target,
                                LOG.CHANNEL_JOIN_SUCCESS)

            # Only send the XDCC message when the first channel was joined
            if not self.channel_joined:

                # Let other on_joins know that message was already sent
                self.channel_joined = True

                log_message = \
                    "Sending XDCC Request to " + self.current_pack.get_bot()
                log_message += \
                    " for pack " + str(self.current_pack.get_packnumber())
                self.logger.log(log_message, LOG.MESSAGE_SEND)

                connection.privmsg(self.current_pack.get_bot(),
                                   self.current_pack.get_request_message())
                # -> on_ctcp

    def on_namreply(self, connection: irc.client.ServerConnection,
                    event: irc.client.Event):
        """
        Event when a channel sends the existing users
        on the channel after joining

        :param connection: the IRC Connection
        :param event:      the IRC Event
        :return:           None
        """
        users = event.arguments[2].split(" ")
        message = "Channel Users:\n"
        for user in users:
            message += user + ";"
        self.logger.log(message, LOG.CHANNEL_USERS)

    def on_endofnames(self, connection: irc.client.ServerConnection,
                      event: irc.client.Event):
        """
        Marks the End of a series of namreplys

        :param connection: the IRC Connection
        :param event:      the IRC Event
        :return:           None
        """
        self.logger.log("End Of joined users list", LOG.CHANNEL_USERS)

    def on_currenttopic(self, connection: irc.client.ServerConnection,
                        event: irc.client.Event):
        """
        Receives the current topic of the newly joined channel

        :param connection: the IRC Connection
        :param event:      the IRC Event
        :return:           None
        """
        self.logger.log("Topic " + str(event.arguments), LOG.CHANNEL_TOPIC)

    def on_topicinfo(self, connection: irc.client.ServerConnection,
                     event: irc.client.Event):
        """
        Receives the current topic information of the newly joined Channel

        :param connection: the IRC Connection
        :param event:      the IRC Event
        :return:           None
        """
        self.logger.log("Topic Info: " + str(event.arguments),
                        LOG.CHANNEL_TOPIC)

    def on_quit(self, connection: irc.client.ServerConnection,
                event: irc.client.Event):
        """
        Logs that a user left a channel

        :param connection: the IRC Connection
        :param event:      the IRC Event
        :return:           None
        """
        self.logger.log(event.arguments[0] + " quit.", LOG.CHANNEL_QUIT)

    def on_part(self, connection: irc.client.ServerConnection,
                event: irc.client.Event):
        """
        Logs that a user left a channel using part

        :param connection: the IRC Connection
        :param event:      the IRC Event
        :return:           None
        """
        self.logger.log(event.source + " left.", LOG.CHANNEL_PART)

    def on_kick(self, connection: irc.client.ServerConnection,
                event: irc.client.Event):
        """
        Logs that a user was kicked from the channel

        :param connection: the IRC Connection
        :param event:      the IRC Event
        :return:           None
        """
        self.logger.log(event.arguments[0] + " was kicked.", LOG.CHANNEL_KICK)

    def on_mode(self, connection: irc.client.ServerConnection,
                event: irc.client.Event):
        """
        Called whenever the mode of a user changes

        :param connection: the IRC Connection
        :param event:      the IRC Event
        :return:           None
        """
        self.logger.log(event.arguments[0] + " mode changed",
                        LOG.CHANNEL_MODE_CHANGE)

    def on_action(self, connection: irc.client.ServerConnection,
                  event: irc.client.Event):
        """
        Called whenever an action on the channel occurs

        :param connection: the IRC Connection
        :param event:      the IRC Event
        :return:           None
        """
        self.logger.log(event.arguments[0] + " mode changed",
                        LOG.CHANNEL_ACTION)

    def on_nick(self, connection: irc.client.ServerConnection,
                event: irc.client.Event):
        """
        Called whenever a user changes his nick

        :param connection: the IRC Connection
        :param event:      the IRC Event
        :return:           None
        """
        self.logger.log(event.source + " nick changed",
                        LOG.CHANNEL_NICK_CHANGED)



"""
Copyright 2016-2017 Hermann Krumrey

This file is part of xdcc-dl.

xdcc-dl is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

xdcc-dl is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with xdcc-dl.  If not, see <http://www.gnu.org/licenses/>.
"""

# imports
import os
from typing import Dict
from xdcc_dl.logging.LoggingTypes import LoggingTypes


class Logger(object):
    """
    Logger for IRC/XDCC related events with variable verbosity levels
    """

    def __init__(self, verbosity_level: int, logfile: str = None,
                 ignore_verbosity_in_logfile: bool = True):
        """
        Initializes a new Logger object.
        The verbosity of the logger is defined here.
        Optionally, a logfile can be used to log to a file
        in addition to the console.
        If a logfile is specified, everything, regardless of verbosity level,
        will be logged to the file by default, but may be changed by toggling
        the ignore_verbosity_in_logfile parameter

        :param verbosity_level: the verbosity level to be used.
                                For more information, see
                                xdcc_dl.logging.LoggingTypes
        :param logfile: The path to a logfile. Specifying this is optional
        :param ignore_verbosity_in_logfile: Determines if the logfile uses
                                            the same verbosity level settings
                                            as the console logging.
                                            By default, the verbosity setting
                                            is ignored.
        """
        self.verbosity_level = verbosity_level
        self.logfile = logfile
        self.ignore_logfile_verbosity = ignore_verbosity_in_logfile

        if self.logfile is not None:
            if not os.path.isfile(self.logfile):
                open(self.logfile, 'w').close()

    def log(self, message: str, logging_type: Dict[str, str or int]=None,
            carriage_return: bool = False):
        """
        Logs a message to the console and optionally to a logfile

        :param message: the message to log
        :param logging_type: the type of message to log
        :param carriage_return: If set to true,
                                appends a carriage return to the log message
        :return: None
        """
        if logging_type is None:
            logging_type = LoggingTypes.DEFAULT

        priority = logging_type['priority']

        if self.logfile is not None:
            if self.ignore_logfile_verbosity or \
                            self.verbosity_level >= priority:
                with open(self.logfile, 'a') as log:
                    log.write(message + "\n")

        if self.verbosity_level >= priority:

            fg_color = logging_type["fg_color"]
            bg_color = logging_type["bg_color"]

            end = "\n" if not carriage_return else "\r"

            print(
                fg_color + bg_color + message + '\033[0m' + '\033[0m', end=end
            )
